@@ -173,14 +173,16 @@ static unsigned char *read_record(dnsheader_t *x, rr_t *y,
      * ... the type of data ...
      */
 
-    y->type = ntohs(*(unsigned short int *) here);
+    memcpy(&y->type, here, sizeof(unsigned short int));
+    y->type = ntohs(y->type);
     here += 2;
 
     /*
      * ... and the class type.
      */
 
-    y->class = ntohs(*(unsigned short int *) here);
+    memcpy(&y->class, here, sizeof(unsigned short int));
+    y->class = ntohs(y->class);
     here += 2;
 
     /*
@@ -193,14 +195,16 @@ static unsigned char *read_record(dnsheader_t *x, rr_t *y,
      * ... while answer blocks carry a TTL and the actual data.
      */
 
-    y->ttl = ntohl(*(unsigned long int *) here);
+    memcpy(&y->ttl, here, sizeof(unsigned long int));
+    y->ttl = ntohl(y->ttl);
     here += 4;
 
     /*
      * Fetch the resource data.
      */
 
-    len = y->len = ntohs(*(unsigned short int *) here);
+    memcpy(&y->len, here, sizeof(unsigned short int));
+    len = y->len = ntohs(y->len);
     here += 2;
     if (y->len > sizeof(y->data) - 4) {
 	y->len = sizeof(y->data) - 4;
@@ -316,10 +320,12 @@ unsigned char *parse_query(rr_t *y, unsigned char *msg, int len)
     k = get_objectname(msg, &here, y->name, 0);
     y->name[k] = 0;
 
-    y->type = ntohs(*(unsigned short int *) here);
+    memcpy(&y->type, here, sizeof(unsigned short int));
+    y->type = ntohs(y->type);
     here += 2;
 
-    y->class = ntohs(*(unsigned short int *) here);
+    memcpy(&y->class, here, sizeof(unsigned short int));
+    y->class = ntohs(y->class);
     here += 2;
 
     k = strlen(y->name);
