@@ -189,12 +189,23 @@ static int reset_master(void)
     int	i;
 
     log_msg(LOG_NOTICE, "resetting master DNS");
-    for (i = 0; i < dbc; i++) {
-    	free_dnsrec(dbv[i]);
+
+    sem_wait(&dnrd_sem);
+
+    for (i = 0; i < dbc; i++) { 
+	free_dnsrec(dbv[i]);
     }
 
-    free(dbv);
-    dbc = 0;
+    if (dbv)
+    {
+	free(dbv);
+	dbc = 0;
+    }
+
+    dbc = 0;    
+    dbmax = 0;  
+
+    sem_post(&dnrd_sem);
 
     return (0);
 }
@@ -301,7 +312,7 @@ int read_hosts(char *filename, char *domain)
 	p = skip_ws(noctrl(line));
 	if (*p == 0  ||  *p == '#') continue;
 
-	if (isdigit(*p)) {
+	if (isdigit((int)(*p))) {
 	    /*
 	     * Usual hosts records start with an IP number.  This
 	     * might be followed by one or more names.  Every
@@ -337,7 +348,7 @@ int read_configuration(char *filename)
 	p = skip_ws(noctrl(line));
 	if (*p == 0  ||  *p == '#') continue;
 
-	if (isdigit(*p)) {
+	if (isdigit((int)(*p))) {
 	    /*
 	     * Usual hosts records start with an IP number.  This
 	     * might be followed by one or more names.  Every
